<?php

namespace FiloBlu\ExtMagentoSalesRule\Model\Rule\Action\Discount;

use Magento\Quote\Model\Quote\Item\AbstractItem;
use Magento\SalesRule\Model\Rule;
use Magento\SalesRule\Model\Rule\Action\Discount\AbstractDiscount;
use Magento\SalesRule\Model\Rule\Action\Discount\Data;

class ByPercentOriginal extends AbstractDiscount
{
    /**
     * @param Rule $rule
     * @param AbstractItem $item
     * @param float $qty
     * @return Data
     */
    public function calculate($rule, $item, $qty)
    {
        $rulePercent = min(100, $rule->getDiscountAmount());
        return $this->_calculate($rule, $item, $qty, $rulePercent);
    }

    /**
     * @param Rule $rule
     * @param AbstractItem $item
     * @param float $qty
     * @param float $rulePercent
     * @return Data
     * @noinspection PhpMissingReturnTypeInspection
     */
    protected function _calculate($rule, $item, $qty, $rulePercent)
    {
        /** @var Data $discountData
         * @noinspection PhpRedundantVariableDocTypeInspection
         */
        $discountData = $this->discountFactory->create();

        $itemPrice = $this->validator->getItemPrice($item);
        $baseItemPrice = $this->validator->getItemBasePrice($item);
        $itemOriginalPrice = $this->validator->getItemOriginalPrice($item);
        $baseItemOriginalPrice = $this->validator->getItemBaseOriginalPrice($item);

        $_rulePct = $rulePercent / 100;

        $fullProductPriceDiscount = (($qty * $itemOriginalPrice - $item->getDiscountAmount()) * $_rulePct);
        $itemPriceRow = ($qty * $itemPrice - $item->getDiscountAmount());
        $itemFullPriceRow = ($qty * $itemOriginalPrice - $item->getDiscountAmount());
        $partialDiscount = ($itemPriceRow - ($itemFullPriceRow - $fullProductPriceDiscount));

        $fullDiscountBase = (($qty * $baseItemOriginalPrice - $item->getBaseDiscountAmount()) * $_rulePct);
        $itemPriceRowBase = ($qty * $baseItemPrice - $item->getBaseDiscountAmount());
        $itemFullPriceRowBase = ($qty * $baseItemOriginalPrice - $item->getBaseDiscountAmount());
        $partialDiscountBase = ($itemPriceRowBase - ($itemFullPriceRowBase - $fullDiscountBase));

        if ($itemPriceRow > ($itemFullPriceRow - $fullProductPriceDiscount)) {
            $discountData->setAmount($partialDiscount);
            $discountData->setBaseAmount($partialDiscountBase);
            $discountData->setOriginalAmount($partialDiscount);
            $discountData->setBaseOriginalAmount($partialDiscountBase);
        } else {
            $discountData->setAmount(0);
            $discountData->setBaseAmount(0);
            $discountData->setOriginalAmount(0);
            $discountData->setBaseOriginalAmount(0);
        }

        if (!$rule->getDiscountQty() || $rule->getDiscountQty() > $qty) {
            $discountPercent = min(100, $item->getDiscountPercent() + $rulePercent);
            $item->setDiscountPercent($discountPercent);
        }

        return $discountData;
    }

    /**
     * @param float $qty
     * @param Rule $rule
     * @return float
     */
    public function fixQuantity($qty, $rule)
    {
        $step = $rule->getDiscountStep();
        if ($step) {
            $qty = floor($qty / $step) * $step;
        }
        return $qty;
    }
}
